/* rtcdate: Set local system time using PCE's RTC emulator.
 *   written and tested in PC/IX, which lacks mktime().
 */

#include <stdio.h>
#include <time.h>
#include <sys/types.h>

struct tm *localtime();

struct {
	unsigned a : 4;
	unsigned b : 4;
	unsigned c : 4;
	unsigned d : 4;
} lo, hi;

/* mktime() is missing in PC/IX */
time_t mktime(ptr)
	struct tm *ptr;
{
	char target[16], attempt[16];
	time_t mid, lbound, ubound;

	lbound = 0;
	ubound = 0x7FFFFFFF;

	sprintf(target, "%04d%02d%02d%02d%02d%02d",
		ptr->tm_year, ptr->tm_mon, ptr->tm_mday,
		ptr->tm_hour, ptr->tm_min, ptr->tm_sec);

	/* do a binary search to get correct timestamp value */
	while(lbound <= ubound) {
		int r;

		mid = ((lbound >> 1) + (ubound >> 1)) | (lbound & ubound & 1);
		ptr = localtime(&mid);
		sprintf(attempt, "%04d%02d%02d%02d%02d%02d",
			ptr->tm_year, ptr->tm_mon, ptr->tm_mday,
			ptr->tm_hour, ptr->tm_min, ptr->tm_sec);
		r = strcmp(target, attempt);
		if(!r) break;
		else if(r < 0) ubound = mid - 1;
		else if(r > 0) lbound = mid + 1;
	}
	return(mid);
}

int main()
{
	time_t tbuf;
	struct tm *new;

	time(&tbuf);
	new = localtime(&tbuf);

	asm("movb ah, *4");
	asm(".byte 0x66, 0x66, 0xcd, 0x1a");
	asm("mov _lo, cx");
	asm("mov _hi, dx");
	new->tm_year = lo.a + 10 * lo.b + 100 * lo.c + 1000 * lo.d - 1900;
	new->tm_mon  = hi.c + 10 * hi.d - 1;
	new->tm_mday = hi.a + 10 * hi.b;

	asm("movb ah, *2");
	asm(".byte 0x66, 0x66, 0xcd, 0x1a");
	asm("mov _lo, cx");
	asm("mov _hi, dx");
	new->tm_hour = lo.c + 10 * lo.d;
	new->tm_min  = lo.a + 10 * lo.b;
	new->tm_sec  = hi.c + 10 * hi.d;

	printf("Setting system time to %04d-%02d-%02d %02d:%02d:%02d: ",
		new->tm_year + 1900, new->tm_mon + 1, new->tm_mday,
		new->tm_hour, new->tm_min, new->tm_sec);
	tbuf = mktime(new);
	printf("%s.\n", stime(&tbuf) ? "permission denied" : "done");

	return(0);
}
